package com.dragoninfo.dcuc.app.config;

import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.interceptor.AsyncUncaughtExceptionHandler;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.AsyncConfigurer;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.util.concurrent.Executor;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * @author huangzqa
 * @date 2020/11/10
 */
@ConfigurationProperties(prefix = "dcuc.app.async.pool")
@EnableAsync
@Configuration
@Slf4j
public class AsyncExecuteThreadPoolConfig implements AsyncConfigurer {

    /**
     * 核心线程池大小
     */
    private int corePoolSize = -1;

    /**
     * 最大线程数
     */
    private int maxPoolSize = -1;

    /**
     * 活跃时间
     */
    private int keepAliveSeconds = 60;

    /**
     * 队列容量
     */
    private int queueCapacity = -1;

    @Override
    public Executor getAsyncExecutor() {
        initDefaultPoolProperties();

        ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
        executor.setCorePoolSize(corePoolSize);
        executor.setMaxPoolSize(maxPoolSize);
        executor.setQueueCapacity(queueCapacity);
        executor.setKeepAliveSeconds(keepAliveSeconds);
        executor.setThreadNamePrefix("dcuc-async-task-");
        executor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        executor.setWaitForTasksToCompleteOnShutdown(true);
        executor.setAwaitTerminationSeconds(60);
        executor.initialize();

        return executor;
    }

    @Override
    public AsyncUncaughtExceptionHandler getAsyncUncaughtExceptionHandler() {
        return (ex, method, params) -> log.error("异步线程调用["
                + method.getDeclaringClass().getName() + "." + method.getName()
                + "]方法出错，参数：" + JsonUtils.toJSONString(params), ex);
    }

    /**
     * 初始化默认线程池参数
     **/
    private void initDefaultPoolProperties() {
        int procNum = Runtime.getRuntime().availableProcessors();
        if (corePoolSize == -1) {
            corePoolSize = procNum;
        }
        if (maxPoolSize == -1) {
            maxPoolSize = 2 * procNum;
        }
        if (queueCapacity == -1) {
            queueCapacity = 10 * maxPoolSize;
        }
    }

    public int getCorePoolSize() {
        return corePoolSize;
    }

    public void setCorePoolSize(int corePoolSize) {
        this.corePoolSize = corePoolSize;
    }

    public int getMaxPoolSize() {
        return maxPoolSize;
    }

    public void setMaxPoolSize(int maxPoolSize) {
        this.maxPoolSize = maxPoolSize;
    }

    public int getKeepAliveSeconds() {
        return keepAliveSeconds;
    }

    public void setKeepAliveSeconds(int keepAliveSeconds) {
        this.keepAliveSeconds = keepAliveSeconds;
    }

    public int getQueueCapacity() {
        return queueCapacity;
    }

    public void setQueueCapacity(int queueCapacity) {
        this.queueCapacity = queueCapacity;
    }
}
